# tartools.py
#
# some maybe useful functions for tarfile users.
#
# (C)2002 Lars Gustbel (lars@gustaebel.de)
# $Id: tartools.py,v 1.2 2002/10/04 18:07:12 lars Exp $

import tarfile
import gzip

#-----------------------------------------------------------------------------
# TarInc/TarIncGz - Create tar archives from virtual files, see TarInc.__doc__
#
# inspired by Ben Escoto
#-----------------------------------------------------------------------------
class FileObj:
    def __init__(self, parent, fileobj):
        self.parent = parent
        self.fileobj = fileobj
    def write(self, buf):
        self.fileobj.write(buf)
    def close(self):
        self.parent._fix()

class GzFileObj:
    def __init__(self, parent, fileobj):
        self.parent = parent
        parent.current.name += ".gz"
        self.fileobj = gzip.GzipFile(parent.current.name, "wb", 9, fileobj)
    def write(self, buf):
        self.fileobj.write(buf)
    def close(self):
        self.fileobj.close()
        self.parent._fix()

class TarInc(tarfile.TarFile):
    """Class that is able to create tar archives from files
       whose sizes are not yet detectable when adding them, e.g.
       the output of a command/pipe or data that is processed
       on-the-fly and is too large to put in memory or in a
       temporary file.

       Example:

       t = TarInc("sample.tar")                 # create a tar archive

       tarinfo = tarfile.TarInfo("testfile")    # prepare to add testfile
       f = t.addfile(tarinfo)                   # we get a file-like object and
       f.write("This is a simple text line!\n" * 1234)  # write our data to it.
       f.close()                                # always close the file object!

       t.close()

    """

    def __init__(self, name):
        """Create a tar archive at path name.
        """
        self.name = name

        fileobj = file(name, "w")
        fileobj.close()
        fileobj = file(name, "r+b")
        self.current = None

        tarfile.TarFile.__init__(self, name, "w", fileobj)

    def _getfileobj(self):
        return FileObj(self, self.fileobj)

    def addfile(self, tarinfo):
        """Add a file whose information lies in tarinfo and
           return a file-object that the data can be written
           to. When the data is complete you must close the
           file-object before you can do any further operation.
        """
        if self.current:
            raise ValueError, "you must close() the previous file object first."

        tarinfo.offset = self.chunks[-1]
        self.fileobj.seek(tarinfo.offset)
        self.fileobj.write("\0" * tarfile.BLOCKSIZE)
        tarinfo.offset_data = self.fileobj.tell()

        self.current = tarinfo
        return self._getfileobj()

    def add(self, name, arcname=None, recursive=1):
        """See TarFile class.
        """
        if self.current:
            raise ValueError, "you must close() the previous file object first."
        tarfile.TarFile.add(self, name, arcname, recursive)

    def _fix(self):
        if self.current is None:
            raise ValueError, "the file has already been closed."

        start = self.current.offset_data
        end   = self.fileobj.tell()
        size  = end - start
        rem   = tarfile.BLOCKSIZE - (size % tarfile.BLOCKSIZE)

        self.fileobj.seek(self.current.offset)
        self.current.size = size
        self.fileobj.write(self.current.getheader())

        self.fileobj.seek(end)
        self.fileobj.write("\0" * rem)
        self.offset = self.fileobj.tell()
        self.chunks.append(self.offset)

        self.current = None

class TarIncGz(TarInc):
    """Same as TarInc but adds the files as gzip
       compressed members with the suffix ".gz".
    """
    def _getfileobj(self):
        return GzFileObj(self, self.fileobj)

#-----------------------------------------------------------------------------
# TarCount  -  calculate size of a tar before creating it.
#-----------------------------------------------------------------------------
class TarCount(tarfile.TarFile):
    """Calculate the final size of tar archive before creating it.
       You may add all your files and directories as usual. When
       finished, the close() method will return the size in bytes.
       (only plain tar, gzip won't work!)
    """

    def __init__(self, name):
        self.name = name
        self._mode = "w"
        self.fileobj = 1
        self.chunks = [0]
        self.inodes = {}
        self.offset = 0l

    def addfile(self, tarinfo, fileobj=None):
        """Count bytes of tarfile.
        """
        if len(tarinfo.linkname) >= tarfile.LENGTH_LINK - 1:
            self.offset += tarfile.BLOCKSIZE * 2
        if len(tarinfo.name) >= tarfile.LENGTH_NAME - 1:
            self.offset += tarfile.BLOCKSIZE * 2

        header = tarinfo.getheader()
        self.offset += tarfile.BLOCKSIZE

        if fileobj is not None:
            blocks, remainder = divmod(tarinfo.size, tarfile.BLOCKSIZE)
            if remainder > 0:
                blocks += 1
            self.offset += blocks * tarfile.BLOCKSIZE
        self.chunks.append(self.offset)

    def close(self):
        """Return the size of the (virtually) created TAR archive,
           as if it were closed. (It is actually NOT closed.)
        """
        blocks, remainder = divmod(self.offset, tarfile.RECORDSIZE)
        if remainder > 0:
            return self.offset + (tarfile.RECORDSIZE - remainder)
        return self.offset
